enyo.kind({
    name: "maklesoft.BrowserApp",
    kind: "BrowserApp",
    create: function() {
        enyo.setFullScreen(true);
        this.createComponent({kind: "HFlexBox", name: "tabsBar", className: "tabsbar", components: [
            {kind: "CustomButton", className: "main-menu-button", onclick: "openAppMenu"},
            {kind: "Scroller", flex: 1, name: "tabsScroller", showing: true, autoVertical: false, autoHorizontal: false, vertical: false,
                layoutKind: "HFlexLayout", components: [
                    {name: "tabs", kind: "RadioToolButtonGroup", className: "tabs", onChange: "tabChanged", value: 0}
            ]},
            {kind: "IconButton", icon: "images/chrome/menu-icon-arrow.png", className: "enyo-button-dark tabs-dropdown-button", toggling: true, onclick: "openTabsMenu", name:"tabsMenuButton"}
        ]});
       
        //enyo.error("*********** version: " + enyo.fetchDeviceInfo().platformVersionDot);
        var devInfo = enyo.fetchDeviceInfo();
        
        // Check if webOS version is at least 3.0.2. If not, hide tabs.
        if (devInfo && devInfo.platformVersionMinor == 0 && devInfo.platformVersionDot < 2) {
            this.$.tabsBar.hide();
            enyo.setFullScreen(false);
        }
        
        this.inherited(arguments);
        this.putPermissions();
        //enyo.keyboard.setResizesWindow(true);
        this.$.applicationEvents.onWindowParamsChange = "windowParamsChangeHandler";
        
        // Use the correct kind for db services
        this.$.bookmarksService.setDbKind(enyo.fetchAppInfo().id + ".browserbookmarks:1");
        this.$.historyService.setDbKind(enyo.fetchAppInfo().id + ".browserhistory:1");
        this.$.browserPrefsService.setDbKind(enyo.fetchAppInfo().id + ".browserpreferences:1");
		
		// Put kinds if not put allready
        this.putDbKinds();
        
		this.setupAppMenu();
	    this.createComponent({
            kind: "Popup", scrim: true, name: "appInfoPopup", components: [{
                kind: "AppInfo", onClose: "closeAppInfo", onOpenVendorSite: "openVendorSite", onOpenTwitter: "openTwitter"
            }]
		});
		
		this.fetchInitialPreferences();
		
		// TODO: fetch last open tabs if exist
		this.tabCount = 0;
		
		this.$.pane.validateView("browser");
//		this.$.browser.onNewTab = "newTab";
		this.$.browser.onOpenStartPage = "openStartPage";
		//this.$.browser.onAddBookmark = "showNewBookmarkDialog";
//		this.$.browser.browserIndex = 0;
		this.$.pane.validateView("startPage");
		this.$.startPage.onDeleteBookmark = "deleteBookmark";
		this.$.startPage.onNewBookmark = "showNewBookmarkDialog";
		this.$.drawerPane.validateView("bookmarks");
		this.$.bookmarks.onAddBookmark = "showNewBookmarkDialog";
		this.$.pane.validateView("preferences");
		this.$.preferences.onHide = "preferencesHidden";
		
		this.$.pane.applyStyle("position", "relative");
		this.$.pane.applyStyle("z-index", "100");
		
		this.loadTabs();
    },
    openStartPage: function() {
        //var browserIndex = this.$.tabs.getValue();
        //this.$["tab" + browserIndex].url = null;
        this.$.pane.selectViewByName("startPage");
    },
    putBookmarksKindSuccess: function() {
        enyo.log("Bookmarks kind put successully!");
    },
    putHistoryKindSuccess: function() {
        enyo.log("History kind put successfully!");
    },
    putBrowserPrefsKindSuccess: function() {
        enyo.log("Browser prefs kind put successfully!");
    },
    windowParamsChangeHandler: function() {
        //this.error("***** window params changed!: " + JSON.stringify(enyo.windowParams));
		var p = enyo.windowParams;
		var url = p.target || p.url;
		if (url) {
			this.setUrl(url); // this opens the browser view
		} else if (p.webviewId) {
	        var browserIndex = this.$.tabs.getValue();
			this.$.pane.selectViewByName("browser" + browserIndex);
			this.$["browser" + browserIndex].setWebViewId(p.webviewId);
		}
        
	    var browserIndex = this.$.tabs.getValue();
        if (enyo.windowParams.credentials && this.$["browser" + browserIndex]) {
            this.$["browser" + browserIndex].openAutoInsertTool(enyo.windowParams.credentials.loginName, enyo.windowParams.credentials.password, enyo.windowParams.title);
        } else {
            this.$["browser" + browserIndex].closeAutoInsertTool();
        }
    },
	handleResource: function(inSender, inMimeType, inUrl) {
		this.downloadResource(inUrl, inMimeType);
	},
	putPermissions: function() {
	    var putPermissions = enyo.create({
            name: "putPermissions", kind: "PalmService", service: "palm://com.palm.db/", method: "putPermissions", onSuccess: "putPermissionsSuccess", onFailure: "serviceFailure", owner: this 
        });
	    putPermissions.call({
	        permissions: [{
                type: "db.kind",
                object: enyo.fetchAppInfo().id + ".browserbookmarks:1",
                caller: "com.palm.launcher",
                operations: {read: "allow"}
            }]
        });
	    putPermissions.call({
	        permissions: [{
                type: "db.kind",
                object: enyo.fetchAppInfo().id + ".browserhistory:1",
                caller: "com.palm.launcher",
                operations: {read: "allow"}
            }]
        });
	},
	putPermissionsSuccess: function() {
	    //enyo.log("***** permissions put successfully!");
	},
	serviceFailure: function(sender, response) {
	    enyo.error("service call failed! " + JSON.stringify(response));
	},
	putDbKinds: function() {
		this.$.bookmarksService.call({
            owner: enyo.fetchAppInfo().id,
	        indexes:[	
	            {"name": "rev","props":[{"name":"_rev"}]},
				{"name": "url", "props": [{"name": "url"}] },
				{"name": "title", "props": [{"name": "title"}] },
				{"name": "search", "props": [{"name":"searchText", "type":"multi","collate":"primary","include":[{"name":"title","tokenize":"all"},{"name":"url","tokenize":"all"}]}]},
				{"name": "defaultEntry", "props": [{"name": "defaultEntry"}] },
				{"name": "idx", "props": [{"name": "idx"}] },
				{"name": "logins", "props": [{"name": "url"},{"name": "hasLogin"}] }
			],
			sync: true
        }, {method: "putKind", onSuccess: "putBookmarksKindSuccess", onFailure: "serviceFailure"});
		this.$.historyService.call({
            owner: enyo.fetchAppInfo().id,
	        indexes: [
	            {"name": "url", "props": [{"name": "url"}] },
				{"name": "title", "props": [{"name": "title"}] },
				{"name": "search", "props": [{"name":"searchText", "type":"multi", "collate":"primary","include":[{"name":"title","tokenize":"all"},
				{"name":"url","tokenize":"all"}]}]},
				{"name": "date", "props": [{"name": "date"}] },
				{"name": "revision", "props": [{"name": "_rev"}] }
	        ],
	        sync: true
        }, {method: "putKind", onSuccess: "putHistoryKindSuccess", onFailure: "serviceFailure"});
		this.$.browserPrefsService.call({
            owner: enyo.fetchAppInfo().id,
	        indexes: [
		        {"name":"key", "props":[{"name":"key"}]}
	        ],
	        sync: true
        }, {method: "putKind", onSuccess: "putBrowserPrefsKindSuccess", onFailure: "serviceFailure"});
	},
	showNewBookmarkDialog: function(sender) {
        var title="", url="";
        var browserIndex = this.$.tabs.getValue();
	    if (this.isBrowserShowing()) {
	        title = this.$["browser" + browserIndex].title;
	        url = this.$["browser" + browserIndex].url;
	    }
		this.$.bookmarkDialog.setTitle(title);
		this.$.bookmarkDialog.setUrl(url);
		this.$.bookmarkDialog.setHasLogin(false);
		//this.$.bookmarkDialog.setIcons({thumbnailFile: inBookmark.thumbnailFile, iconFile32: inBookmark.iconFile32, iconFile64: inBookmark.iconFile64});
		//this.$.bookmarkDialog.setId(inBookmark._id);
		this.$.bookmarkDialog.setAcceptCaption($L("Save"));
		this.$.bookmarkDialog.acceptAction = "newBookmark";
		this.$.bookmarkDialog.openAtCenter();
	},
	newBookmark: function(title, url) {
		var date = (new Date()).getTime();
		var b = {
			_kind: this.$.bookmarksService.dbKind,
			title: title,
			url: url,
			date: date,
			lastVisited: date,
			defaultEntry: false,
			visitCount: 0,
			idx: null
		};
		this.$.bookmarksService.call({objects: [b]}, {method: "put"});
	},
	tabChanged: function(sender) {
	    if (sender.getValue() == "newTab") {
	        this.newTab();
	        return;
	    }
	    var browserIndex = sender.getValue() || 0;
	    //enyo.log("selecting tab " + browserIndex);
	    var tab = this.$["tab" + browserIndex]
	    var url = tab.url;
	    this.$.tabsScroller.scrollIntoView(tab.getBounds().top, tab.getBounds().left);
        if (url) {
            this.$.pane.selectViewByName("browser" + browserIndex);
        } else {
            this.$.pane.selectViewByName("startPage");
        }
	},
	newTab: function(inSender, inEvent) {
	    //enyo.log("***************** new Tab: " + JSON.stringify(inEvent));
	    inEvent = inEvent || {};
	    inEvent.title = inEvent.title || $L("New Tab");
	    this.tabCount = this.tabCount || 0;
	    var browserIndex = this.tabCount;
	    this.tabCount++;
	    if (this.$.newTab) {
	        this.$.newTab.destroy();
	    }
		var tab = this.$.tabs.createComponent({
		    name: "tab" + browserIndex, value: browserIndex, title: inEvent.title, url: inEvent.url, owner: this, allowDrag: true,
		    layoutKind: "HFlexLayout", align: "start", className: "enyo-radiobutton-dark enyo-single", components: [
		        {name: "tabTitle" + browserIndex, content: inEvent.title, flex: 1, className: "tab-text"}, 
		        {name: "tabCloseButton" + browserIndex, kind: "CustomButton", className: "tab-close-button", onclick: "closeTab"}
		    ]
		});
//		if (this.$.tabs.getControls().length == 1) {
//		    this.$["tabCloseButton" + browserIndex].hide();
//		}
		this.$.tabs.createComponent({
		    name: "newTab", value: "newTab", title: $L("Create New Tab"), caption: "+", owner: this, className: "enyo-radiobutton-dark enyo-single", style: "width: 60px !important; font-size: 13pt"
		});
		this.$.tabs.render();
		this.$.tabs.setValue(browserIndex);
		if (this.tabCount > 1) {
		    this.$.tabs.show();
		}            
		
		this.$.pane.createComponent({
            owner: this,
            name: "browser" + browserIndex,
            browserIndex: browserIndex, 
            kind: "Browser",
			onPageTitleChanged: "pageTitleChanged",
			onPageLoadStopped: "pageLoadStopped",
			onFileLoad: "handleResource",
			onAddBookmark: "addBookmark",
			onAddToLauncher: "showAddtoLauncherDialog",
			onShareLink: "shareClick",
			onOpenBookmarks: "bookmarksClick",
			onPrint: "printClick",
			onClose: "browserClosed",
			onShow: "browserShown",
			onHide: "browserHidden",
		    onNewTab: "newTab",
		    onOpenStartPage: "openStartPage",
		    onShowTabs: "showTabs",
		    onHideTabs: "hideTabs"
		    //onAddBookmark: "showNewBookmarkDialog"
		});
		this.$["browser" + browserIndex].render();
		
		if (inEvent.url) {
		    this.$.pane.selectViewByName("browser" + browserIndex);
		    this.$["browser" + browserIndex].setUrl(inEvent.url);
		} else if (inEvent.identifier) {
		    this.$.pane.selectViewByName("browser" + browserIndex);
		    this.$["browser" + browserIndex].setWebViewId(inEvent.identifier);
		} else {
		    this.$.pane.selectViewByName("startPage");
		}
		return tab;
	},
	closeTab: function(sender) {
	    var browserIndex = sender.parent.value;
	    if (this.$.tabs.getControls().length > 2) {
	        this.$["tab" + browserIndex].destroy();
	        this.$["browser" + browserIndex].destroy();
		    var currentIndex = this.$.tabs.getValue();
		    if (browserIndex == currentIndex) {
		        if (this.$["tab" + (currentIndex - 1)]) {
		            this.$.tabs.setValue(currentIndex - 1);
		        } else {
		            this.$.tabs.setValue(currentIndex + 1);
		        }
		        this.tabChanged(this.$.tabs);
		    }
		}
	    return true;
	},
	setupAppMenu: function() {
	    this.$.appMenu.destroy();
	    
	    this.createComponent({kind: "AppMenu", name: "appMenu", onBeforeOpen: "toggleAppMenuItems", components: [
			{name: "preferencesItem", caption: $L("Preferences"), onclick: "preferencesClick"},
			{name: "printMenuItem", caption: $L("Print"), onclick: "printClick"},
			{caption: $L("New Card"), onclick: "newCardClick"},
			{caption: $L("Bookmarks"), onclick: "openBookmarks"},
			{caption: $L("History"), onclick: "openHistory"},
			{caption: $L("Downloads"), onclick: "openDownloads"},
			{caption: $L("About"), onclick: "showAppInfo"}
		]});
	},
	showAppInfo: function() {
	    this.$.appInfoPopup.openAtCenter();
	},
	closeAppInfo: function() {
	    this.$.appInfoPopup.close();
	},
	openVendorSite: function() {
	    this.setUrl("http://www.maklesoft.com");
	    this.closeAppInfo();
	},
	openTwitter: function() {
	    this.setUrl("http://www.twitter.com/maklesoft");
	    this.closeAppInfo();
	},
	showTabs: function(showing) {
	    this.$.tabsBar.show();
	},
	hideTabs: function(showing) {
	    this.$.tabsScroller.hide();
	},
	openAppMenu: function() {
	    this.$.appMenu.open();
	},
	openBookmarks: function() {
	    this.$.toaster.open();
	    this.showBookmarks();
	},
	openHistory: function() {
	    this.$.toaster.open();
	    this.showHistory();
	},
	openDownloads: function() {
	    this.$.toaster.open();
	    this.showDownloads();
	},
	openTabsMenu: function(sender) {
	    if (this.$.tabsMenu) {
	        this.$.tabsMenu.destroy();
	    }
	    var items = [];
	    var tabs = this.$.tabs.getControls();
	    
	    for (var i = 0; i < tabs.length; i++) {
	        items.push({caption: tabs[i].title, value: tabs[i].getValue(), onclick: "tabSelected"});
	    }
	    enyo.log("**** tabs: " + JSON.stringify(items));
	    this.createComponent({kind: "Menu", name: "tabsMenu", components: items, onClose: "tabsMenuClosed"});
	    this.$.tabsMenu.openAroundControl(sender);
	},
	tabsMenuClosed: function() {
	    this.$.tabsMenuButton.setDepressed(false);
	},
	tabSelected: function(sender) {
	    this.$.tabs.setValue(sender.value);
	    this.tabChanged(this.$.tabs);
	},
	loadTabs: function() {
//	    var tabsString = enyo.getCookie("open_tabs");
//	    try {
//	        tabs = JSON.parse(tabsString);
//	    } catch(e) {
//	        tabs = [];
//	        this.error(e);
//	    }
//	    this.preloadedTabs = [];
//	    for (var i = 0; i < tabs.length; i++) {
//	        this.preloadedTabs.push(this.newTab(this, tabs[i]));
//	    }
	    //if (this.url || !tabs.length) {
		    this.newTab(this, {url: this.url, title: this.title});
		//}
	},
	// overwritten
	applicationRelaunchHandler: function(inSender) {
	    //this.error("application relaunched!");
		var c = enyo.windows.getActiveWindow();
		var params = enyo.windowParams;
		// squelch needless banner clicks
		if (params.dontLaunch) return true;
		if (params.sendDataToShare !== undefined) {
			this.$.stService.call({data: {target: c.enyo.$.browserApp.getUrl(), type: "rawdata", mimetype: "text/html"}}, {method: "shareData"});
			return true;
		} else if (params.toasterOpen) {
			this.$.toaster.open();
			this.$.radioGroup.setValue(params.toasterOpen);
			this.$.drawerPane.selectViewByPane(params.toasterOpen);
		}
	},
//	rendered: function() {
//		this.inherited(arguments);
//		this.log();
//		var p = enyo.windowParams;
//		var url = p.target || p.url;
//		this.setUrl(url);
//		this.showBookmarks();
//	},
	isBrowserShowing: function() {
		return !(this.isStartPageShowing() || this.isPreferencesShowing());
	},
	isBrowserLoading: function() {
	    var browserIndex = this.$.tabs.getValue();
		if (this.$["browser" + browserIndex]) {
			return this.$["browser" + browserIndex].isLoading();
		} else {
			return false;
		}
	},
	resizeHandler: function() {
		this.inherited(arguments);
	    var browserIndex = this.$.tabs.getValue();
		if (this.isBrowserShowing()) {
			this.$["browser" + browserIndex].resize();
		} else if (this.isStartPageShowing()) {
			this.$.startPage.resize();
		}
	},
	windowActivatedHandler: function() {
		this.lastActiveWindow = enyo.windows.getActiveWindow();
//		if (this.lastActiveWindow != enyo.windows.getRootWindow()){
//		    for (var i = 0; i < this.preloadedTabs.length; i++) {
//		        enyo.log("************* closing tab: " + this.preloadedTabs[i]);
//		        this.closeTab(this.preloadedTabs[i]);
//		    }
//		}
	    var browserIndex = this.$.tabs.getValue();
		if (this.isBrowserShowing()) {
			this.$["browser" + browserIndex].viewCall("activate");
		}
		this.refreshDownloads();
	},
	windowDeactivatedHandler: function() {
	    //enyo.log("**************** window is root: " + (this.lastActiveWindow == enyo.windows.getRootWindow()));
		//remember tabs if window is root
		if (this.lastActiveWindow == enyo.windows.getRootWindow()) {
		    var tabs = this.$.tabs.getControls();
		    var tabsList = [];
		    for (var i = 0; i < tabs.length-1; i++) {
		        tabsList.push({url: tabs[i].url, title: tabs[i].title});
		    }
		    enyo.setCookie("open_tabs", JSON.stringify(tabsList));
		}
		
	    var browserIndex = this.$.tabs.getValue();
		if (this.isBrowserShowing()) {
			this.$["browser" + browserIndex].viewCall("deactivate");
		}
	},
	browserShown: function(inSender, inShowing, inRefresh) {
	    var browserIndex = this.$.tabs.getValue();
	    //enyo.error("browser shown: " + browserIndex);
	    this.showingBrowserIndex = browserIndex;
		enyo.keyboard.setResizesWindow(false);
		//this.$["browser" + browserIndex].viewCall("activate");
		if (inRefresh || !this.$["browser" + browserIndex].getUrl()) {
			this.$["browser" + browserIndex].setUrl(this.url);
		}
	},
	browserHidden: function() {
	    //var browserIndex = this.$.tabs.getValue();
//	    if (this.showingBrowserIndex != null && this.showingBrowserIndex != undefined) {
		    enyo.keyboard.setResizesWindow(true);
//		    this.$["browser" + this.showingBrowserIndex].viewCall("deactivate");
//		}
	},
	applyPreference: function(inPreference, inValue) {
	    var browserIndex = this.$.tabs.getValue();
		this.log(inPreference, inValue);
		var preferenceMap = {
			enableJavascript: "setEnableJavascript",
			blockPopups: "setBlockPopups",
			acceptCookies: "setAcceptCookies"
		}
		this.$.pane.viewByName("browser" + browserIndex);
		if (inPreference == "clearHistory" || inPreference == "clearBookmarks") {
			this[preferenceMap[inPreference] || inPreference](inValue);
		} else {
		    for (var i = 0; i < this.tabCount; i++) {
		        var o = this.$["browser" + browserIndex];
		        if (o && (o[preferenceMap[inPreference] || inPreference])) {
			        o[preferenceMap[inPreference] || inPreference](inValue);
		        }
		    }
		}
	},
	searchPreferencesChanged: function() {
	    var browserIndex = this.$.tabs.getValue();
		var views = ["browser", "startPage", "preferences"];
	    for (var i = 0; i < this.tabCount; i++) {
	        views.push("browser" + browserIndex);
	    }
		for (var i=0; i < views.length; i++) {
		    if (this.$.pane.viewByName(views[i])) {
			    this.$.pane.viewByName(views[i]).setSearchPreferences(this.searchPreferences);
			};
		}
	},
	defaultSearchChanged: function() {
	    var browserIndex = this.$.tabs.getValue();
		this.log(this.defaultSearch);
		var views = ["browser", "startPage", "preferences"];
	    for (var i = 0; i < this.tabCount; i++) {
	        views.push("browser" + browserIndex);
	    }
		for (var i=0; i < views.length; i++) {
		    if (this.$.pane.viewByName(views[i])) {
			    this.$.pane.viewByName(views[i]).setDefaultSearch(this.defaultSearch);
			};
		}
	},
    urlChanged: function() {
        if (this.url) {
	        var browserIndex = this.$.tabs.getValue();
	        //enyo.log("url changed! url: " + this.url + "; browserIndex: " + browserIndex); 
	        this.$.pane.selectViewByName("browser" + browserIndex);
	        this.$["browser" + browserIndex].setUrl(this.url);
	    } else {
	        this.$.pane.selectViewByName("startPage");
	    }
    },
	pageTitleChanged: function(inSender, inTitle, inUrl) {
	    this.inherited(arguments, [inSender, inTitle, inUrl]);
	    var index = inSender.browserIndex || 0;
	    if (this.$["tab" + index]) {
	        this.$["tabTitle" + index].setContent(inTitle);
	        this.$["tab" + index].url = inUrl;
	        this.$["tab" + index].title = inTitle;
	    }
	},
	renderDocument: function(inSender, inJobID, inPrintParams) {
		if (this.isBrowserShowing()) {
	        var browserIndex = this.$.tabs.getValue();
			this.$["browser" + browserIndex].printFrame(inJobID, inPrintParams);
		}
	},
	showEditBookmarkDialog: function(inSender, inBookmark) {
	    this.inherited(arguments, [inSender, inBookmark]);
	    this.$.bookmarkDialog.setHasLogin(inBookmark.hasLogin);
	    this.$.bookmarkDialog.setCredentials(inBookmark.credentials);
	},
	bookmarkAccept: function(inSender, inTitle, inUrl, inIcons, inId, inHasLogin, inCredentials) {
		var a = inSender.acceptAction;
		if (this[a]) {
			this[a](inTitle, inUrl, inIcons, inId, inHasLogin, inCredentials);
		}
	},
	editBookmark: function(inTitle, inUrl, inIcons, inId, inHasLogin, inCredentials) {
		var date = (new Date()).getTime();
		var b = {
			_id: inId,
			title: inTitle,
			url: inUrl,
			date: date,
			hasLogin: inHasLogin,
		    credentials: inCredentials
		};
		enyo.mixin(b, inIcons);
		this.$.bookmarksService.call({objects: [b]}, {method: "merge"});
	},
	preferencesClick: function() {
		this.gotoView("preferences");
	},
	backHandler: function(inSender, e) {
		if (this.isBrowserShowing()) {
	        var browserIndex = this.$.tabs.getValue();
			this.$["browser" + browserIndex].goBack();
			e.preventDefault();
		} else if (!this.isStartPageShowing()) {
			this.$.pane.back(e);
		}
	},
	addBookmark: function(sender, edit) {
	    if (edit) {
	        this.showNewBookmarkDialog();
	    } else {
		    var date = (new Date()).getTime();
	        var browserIndex = this.$.tabs.getValue();
		    var b = {
			    _kind: this.$.bookmarksService.dbKind,
			    title: this.$["browser" + browserIndex].title,
			    url: this.$["browser" + browserIndex].url,
			    date: date,
			    lastVisited: date,
			    defaultEntry: false,
			    visitCount: 0,
			    idx: null
		    };
		    enyo.mixin(b, this.$["browser" + browserIndex].createPageImages());
		    this.$.bookmarksService.call({objects: [b]}, {method: "put"});
		}
	},
	preferencesShown: function() {
	    this.inherited(arguments);
	    this.$.tabs.hide();
	    this.$.tabsMenuButton.hide();
	},
	preferencesHidden: function() {
	    this.$.tabs.show();
	    this.$.tabsMenuButton.show();
	}
});
